from openai import OpenAI
from gradio_agentchatbot import AgentChatbot
import gradio as gr

from dotenv import load_dotenv

load_dotenv()


client = OpenAI()


def openai_stream_response(prompt: str, history):
    history.append({"role": "user", "content": prompt})
    yield history

    response = client.chat.completions.create(
        model="gpt-4o", messages=history, stream=True
    )

    message = {"role": "assistant", "content": ""}
    for chunk in response:
        message["content"] += chunk.choices[0].delta.content or ""
        yield history + [message]


with gr.Blocks() as demo:
    gr.Markdown("# Chat with GPT-4o")
    chatbot = AgentChatbot(
        label="Agent",
        avatar_images=(
            None,
            "https://huggingface.co/spaces/KingNish/OpenGPT-4o/resolve/main/OpenAI_logo.png",
        ),
    )
    prompt = gr.Textbox(lines=1, label="Chat Message")
    prompt.submit(openai_stream_response, [prompt, chatbot], [chatbot])


if __name__ == "__main__":
    pass



import gradio as gr
from transformers import load_tool, ReactCodeAgent, HfEngine, Tool
from gradio_agentchatbot import (
    AgentChatbot,
    stream_from_transformers_agent,
    Message,
)
from dotenv import load_dotenv
from langchain_community.tools.tavily_search import TavilySearchResults
from pathlib import Path

current_dir = Path(__file__).parent

load_dotenv()

image_generation_tool = load_tool("m-ric/text-to-image")

search_tool = TavilySearchResults()

llm_engine = HfEngine("meta-llama/Meta-Llama-3-70B-Instruct")
# Initialize the agent with both tools
agent = ReactCodeAgent(
    tools=[image_generation_tool, search_tool], llm_engine=llm_engine
)


def interact_with_agent(prompt, messages):
    messages.append(Message(role="user", content=prompt))
    yield messages
    for msg in stream_from_transformers_agent(agent, prompt):
        messages.append(msg)
        yield messages
    yield messages


with gr.Blocks() as demo:
    gr.Markdown("# Chat with an LLM Agent 🤖 and see its thoughts 💭")
    chatbot = AgentChatbot(
        label="Agent",
        avatar_images=(
            None,
            "https://em-content.zobj.net/source/twitter/53/robot-face_1f916.png",
        ),
    )
    prompt = gr.Textbox(lines=1, label="Chat Message")
    prompt.submit(interact_with_agent, [prompt, chatbot], [chatbot])


if __name__ == "__main__":
    demo.launch()



import gradio as gr
from transformers import AutoModelForCausalLM, AutoTokenizer, TextIteratorStreamer
from threading import Thread

# tokenizer = AutoTokenizer.from_pretrained("mistralai/Mistral-7B-Instruct-v0.3")
# model = AutoModelForCausalLM.from_pretrained("mistralai/Mistral-7B-Instruct-v0.3", device_map="auto")
# terminators = [
#     tokenizer.eos_token_id,
#     tokenizer.convert_tokens_to_ids("<|eot_id|>")
# ]

def chat_mistral7b_v0dot3(message: str, 
              history: list, 
              temperature: float, 
              max_new_tokens: int
             ):
    history.append({"role": "user", "content": message})

    input_ids = tokenizer.apply_chat_template(history, return_tensors="pt").to(model.device)
    
    streamer = TextIteratorStreamer(tokenizer, timeout=10.0, skip_prompt=True, skip_special_tokens=True)

    generate_kwargs = dict(
        input_ids= input_ids,
        streamer=streamer,
        max_new_tokens=max_new_tokens,
        do_sample=True,
        temperature=temperature,
        eos_token_id=terminators,
    )
    # This will enforce greedy generation (do_sample=False) when the temperature is passed 0, avoiding the crash.             
    if temperature == 0:
        generate_kwargs['do_sample'] = False
        
    t = Thread(target=model.generate, kwargs=generate_kwargs)
    t.start()

    for text in streamer:
        yield text
        

# Gradio block
with gr.Blocks() as demo:
    gr.Markdown("# ChatInterface with Mistral and Transformers 🤗")
    gr.ChatInterface(
        fn=chat_mistral7b_v0dot3,
        fill_height=True,
        additional_inputs_accordion=gr.Accordion(label="⚙️ Parameters", open=False, render=False),
        additional_inputs=[
            gr.Slider(minimum=0,
                      maximum=1, 
                      step=0.1,
                      value=0.95, 
                      label="Temperature", 
                      render=False),
            gr.Slider(minimum=128, 
                      maximum=4096,
                      step=1,
                      value=512, 
                      label="Max new tokens", 
                      render=False ),
            ],
    )
    
  
if __name__ == "__main__":
    demo.launch()


from huggingface_hub import InferenceClient
from gradio_agentchatbot import ChatInterface, AgentChatbot
import gradio as gr

client = InferenceClient("HuggingFaceH4/zephyr-7b-beta")


def respond(
    message,
    history: list[dict],
    system_message: str,
):
    messages = [{"role": "system", "content": system_message}] + history
    messages.append({"role": "user", "content": message})

    for chunk in client.chat_completion(
        messages,
        stream=True,
    ):
        yield chunk.choices[0].delta.content


chat = ChatInterface(
    respond,
    chatbot=AgentChatbot(height=500),
    additional_inputs=[
        gr.Textbox(value="You are a friendly Chatbot.", label="System message"),
    ],
)

with gr.Blocks() as demo:
    gr.Markdown("# ChatInterface with Hugging Face Zephyr 7b 🤗")
    chat.render()


if __name__ == "__main__":
    demo.launch()



import gradio as gr
from pathlib import Path
from dotenv import load_dotenv

load_dotenv()


from hf_agent_demo import demo as hf_agent
from hf_chatinterface import demo as hf_chatinterface
from langchain_agent_demo import demo as langchain_agent
from openai_stream_demo import demo as openai_stream
from open_ai_function_calling_demo import demo as openai_agent
from transformers_local import demo as transformers_local


with gr.Blocks() as demo:
    with gr.Tabs():
        for file_name, sub_demo, name in [
            ("hf_chatinterface", hf_chatinterface, "Inference API 🤗"),
            ("transformers_local", transformers_local, "Transformers Local 🤗"),
            ("openai_stream_demo", openai_stream, "OpenAI API Chatbot"),
            ("hf_agent_demo", hf_agent, "Transformers Agent 🤖"),
            ("langchain_agent_demo", langchain_agent, "LangChain Agent 🦜⛓️"),
            ("open_ai_function_calling_demo", openai_agent, "OpenAI Function Calling"),
        ]:
            with gr.Tab(name):
                with gr.Tabs():
                    with gr.Tab("Demo"):
                        sub_demo.render()
                    with gr.Tab("Code"):
                        gr.Code(
                            value=Path(f"{file_name}.py").read_text(), language="python"
                        )


if __name__ == "__main__":
    demo.launch()


from .agentchatbot import AgentChatbot, ChatbotData
from .utils import (
    stream_from_transformers_agent,
    Message,
    Metadata,
    MessageDict,
)
from .chat_interface import ChatInterface

__all__ = [
    "AgentChatbot",
    "ChatbotData",
    "stream_from_transformers_agent",
    "Metadata",
    "Message",
    "MessageDict",
    "ChatInterface",
]



from openai import OpenAI
import gradio as gr
from gradio_agentchatbot import AgentChatbot
from typing import Any
import json
from dotenv import load_dotenv

load_dotenv()

client = OpenAI()


# Example dummy function hard coded to return the same weather
# In production, this could be your backend API or an external API
def get_current_weather(location, unit="fahrenheit"):
    """Get the current weather in a given location"""
    if "tokyo" in location.lower():
        return json.dumps({"location": "Tokyo", "temperature": "10", "unit": unit})
    elif "san francisco" in location.lower():
        return json.dumps(
            {"location": "San Francisco", "temperature": "72", "unit": unit}
        )
    elif "paris" in location.lower():
        return json.dumps({"location": "Paris", "temperature": "22", "unit": unit})
    else:
        return json.dumps({"location": location, "temperature": "unknown"})


tools = [
    {
        "type": "function",
        "function": {
            "name": "get_current_weather",
            "description": "Get the current weather in a given location",
            "parameters": {
                "type": "object",
                "properties": {
                    "location": {
                        "type": "string",
                        "description": "The city and state, e.g. San Francisco, CA",
                    },
                    "unit": {"type": "string", "enum": ["celsius", "fahrenheit"]},
                },
                "required": ["location"],
            },
        },
    }
]


def gpt4o_function_calling(prompt: str, history: list):
    # Step 1: send the conversation and available functions to the model
    messages: list[Any] = [m for m in history]
    messages.append({"role": "user", "content": prompt})

    history.append({"role": "user", "content": prompt})
    yield history

    response = client.chat.completions.create(
        model="gpt-4o",
        messages=messages,
        tools=tools,
        tool_choice="auto",  # auto is default, but we'll be explicit
    )
    response_message = response.choices[0].message
    messages.append(response_message)
    if response_message.content:
        history.append(response_message)
        yield history

    tool_calls = response_message.tool_calls
    # Step 2: check if the model wanted to call a function
    if tool_calls:
        # Step 3: call the function
        # Note: the JSON response may not always be valid; be sure to handle errors
        available_functions = {
            "get_current_weather": get_current_weather,
        }  # only one function in this example, but you can have multiple
        # Step 4: send the info for each function call and function response to the model
        for tool_call in tool_calls:
            function_name = tool_call.function.name
            function_to_call = available_functions[function_name]
            function_args = json.loads(tool_call.function.arguments)
            function_response = function_to_call(
                location=function_args.get("location"),
                unit=function_args.get("unit"),
            )
            history.append(
                dict(
                    role="assistant",
                    content=function_response,
                    metadata={"tool_name": function_name},
                )
            )
            yield history
            messages.append(
                {
                    "tool_call_id": tool_call.id,
                    "role": "tool",
                    "name": function_name,
                    "content": function_response,
                }
            )  # extend conversation with function response
        second_response = client.chat.completions.create(
            model="gpt-4o",
            messages=messages,
        )  # get a new response from the model where it can see the function response
        history.append(second_response.choices[0].message)
        yield history


with gr.Blocks() as demo:
    gr.Markdown("# Chat with GPT-4o")
    chatbot = AgentChatbot(
        label="Agent",
        avatar_images=(
            None,
            "https://huggingface.co/spaces/KingNish/OpenGPT-4o/resolve/main/OpenAI_logo.png",
        ),
    )
    prompt = gr.Textbox(lines=1, label="Chat Message")
    prompt.submit(gpt4o_function_calling, [prompt, chatbot], [chatbot])
